/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * die Repräsentation einer WennAnweisung.<br>
 * Eine WennAnweisung in Uda hat eine Menge von potentiell auszuführenden
 * Anweisungsblöcken, die bei einer gültigen Bedingung ausgeführt werden.
 * Optional existiert ein Anweisungsblock, der immer ausgeführt wird, wenn keine
 * Bedingung eines anderen Anweisungsblockes gültig ist.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class WennAnweisung extends AbstractAnweisung {

	/**
	 * die Liste der bedingten Anweisungsblöcke. Die Reihenfolge wird bei der
	 * Erstellung des Objekts aus dem vom Parser gelieferten AST festgelegt. Es
	 * wird implizit vorausgesetzt das als erstes die "DANN", dann alle
	 * folgenden "WENN-DANN"-Alternativen und zuletzt die "SONST"-Alternative in
	 * der Liste eingeordnet sind.
	 */
	private final List<BedingterAnweisungsBlock> alternativen;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung deifniert ist.
	 * @param skriptname
	 *            der Name des Skripts
	 * @param alternativen
	 *            die Liste der ausführbaren Alternativen
	 */
	public WennAnweisung(final int zeile, final String skriptname, final List<BedingterAnweisungsBlock> alternativen) {
		super(zeile, skriptname);
		this.alternativen = alternativen;
	}

	/**
	 * führt die Anweisung aus.<br>
	 * Bei der Anweisung wird schrittweise für jede Alternative in der Liste
	 * ausgeführt. Liefert die Ausführung einer Alternative den logischen Wert
	 * "wahr", d.h. die Bedingung der Alternative ist gültig und die Anweisungen
	 * wurden ausgeführt, dann wird die weitere Ausführung der alternativen
	 * Anweisungsblöcke abgebrochen. <br>
	 * Tritt eine nicht behandelte Ausnahme auf, wird die Ausführung der
	 * Anweisung abgebrochen und die Ausnahme weitergereicht.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public final Object interpret(final Kontext kontext) {
		Object result = null;
		try {
			for (final BedingterAnweisungsBlock alternative : alternativen) {
				if (alternative.isGueltig(kontext)) {
					result = alternative.interpret(kontext);
					break;
				}
			}
		} catch (final InterpreterException e) {
			throw erzeugeAnweisungsFehler(e);
		}

		if (result == null) {
			result = NichtWert.NICHTWERT;
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		return new ArrayList<>(alternativen);
	}
}
