/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.RelativeZeitLiteral;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * die Repräsentation einer Anweisung zur Realisierung einer Zyklus.<br>
 * Die Anweisung initiiert einen Zyklus innerhalb eines Uda-Interpreters, d.h.
 * eine nebenläufige Aktivität, die zyklisch in einem gegebenen Zeitintervall
 * wiederholt wird.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZyklischeAnweisung extends AbstractAnweisung {

	/**
	 * die Zielvariable, in der die Aktivität im Kontext des Anweisungsblocks,
	 * in dem die Aktivität gestartet wird, eingetragen wird.
	 */
	private final Variable variable;

	/**
	 * der Ausdruck zur Definition der Zykluszeit für die Ausführung.
	 */
	private final RelativeZeitLiteral intervall;

	/**
	 * die Anweisungen, die innerhalb eines Zyklus ausgeführt werden sollen.
	 */
	private final AnweisungsBlock anweisungen;

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung definiert ist
	 * @param skript
	 *            der Name des Skripts
	 * @param variable
	 *            die optionale Variable, in der die Aktivität im Kontext
	 *            registriert wird
	 * @param intervall
	 *            die Zykluszeit
	 * @param anweisungen
	 *            die Liste der auszuführenden Anweisungen
	 */
	public ZyklischeAnweisung(final int zeile, final String skript, final Variable variable,
			final RelativeZeitLiteral intervall, final AnweisungsBlock anweisungen) {
		super(zeile, skript);
		this.variable = variable;
		this.intervall = intervall;
		this.anweisungen = anweisungen;
	}

	/**
	 * führt die Anweisung aus.<br>
	 * Mit der Ausführung der Anweisung wird ein Zyklusobjekt als Aktivität
	 * innerhalb des Interpreters angelegt. Das erzeugte Objekt wird optional im
	 * Kontext des Aufrufers unter dem gegebenen Variablennamen abgelegt.<br>
	 * Eine potentiell auftretende Ausnahme beim Starten der Aktivität wird als
	 * UdaFehler weitergereicht.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public final Object interpret(final Kontext kontext) {
		String name = null;
		if (variable != null) {
			name = variable.getName();
		}
		Zyklus zyklus = null;
		try {
			zyklus = new Zyklus((SkriptKontext) kontext, name, intervall.interpret(kontext), anweisungen);
		} catch (final InterpreterException e) {
			throw erzeugeAnweisungsFehler(e);
		}

		return zyklus;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(variable);
		liste.add(intervall);
		liste.add(anweisungen);
		return liste;
	}
}
