/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.anweisung.AbstractAnweisung;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Variable;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * die Repräsentation einer Anmeldeanweisung in einem Uda-Skript.<br>
 * Eine Anmeldeanweisung beschreibt die Installation einer nebenläufigen
 * Aktivität innerhalb des Uda-Interpreters mit der eine Verbindung zu einem
 * Datenverteiler hergestellt werden kann. Über die Verbindung können Daten mit
 * dem Datenverteiler ausgetaucht und innerhalb der Aktivität weiterverarbeitet
 * werden.<br>
 * Die Anmeldeanweisung setzt sich aus verschiedenen Bereichen zusammen:
 * <ul>
 * <li><b>Definitionsbereich:</b> Anweisungen die vor der Anmeldung von
 * Quellen,Senken,Empfängern und Sendern ausgeführt werden. Dieser Bereich wird
 * in erster Linie dazu benötigt, die internen Variablen der Anweisung zu
 * initialisieren</li>
 * <li><b>Anmeldungsbereich:</b> Ausdrücke für die Anmeldung von Kombinationen
 * aus Objekten, Attributgruppen und Aspekten bzw. die Anmeldung von Mengen
 * <ul>
 * <li>Quellenanmeldungen</li>
 * <li>Senderanmeldungen</li>
 * <li>Senkenanmeldungen</li>
 * <li>Empfängeranmeldungen</li>
 * </ul>
 * </li>
 * <li><b>Fehlerbehandlungsbereich:</b> Anweisungen, die ausgeführt werden, wenn
 * unbehandelte Fehler in einem anderen Bereich der Anweisung auftreten</li>
 * <li><b>Initialbereich:</b> Anweisungen die einmalig ausgeführt werden,
 * nachdem alle Anmeldungen erfolgt sind</li>
 * <li><b>Aktualisierungbereich:</b> Anweisungen, die beim Empfang von Daten vom
 * Datenverteiler ausgeführt werden. Dieser Bereich entspricht der <i>update</i>
 * -Funktion der Datenverteiler-Schnittstelle <i>ClientReceiverInterface</i>
 * </li>
 * <li><b>Mengenänderungbereich:</b> Anweisungen, die ausgeführt werden, wenn
 * eine dynamische Menge verändert wurde. Diese Funktion entspricht der
 * <i>update</i>-Funktion der Datenverteiler-Schnittstelle
 * <i>MutableSetChangeListener</i></li>
 * </ul>
 * Die Bereiche sind optional, lediglich ihre Reihenfolge ist festgelegt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class AnmeldeAnweisung extends AbstractAnweisung {

	/**
	 * der Name der Variable unter der die Anmeldeanweisung im umgebenden
	 * Kontext abgelegt wird.
	 */
	private final String name;

	/**
	 * die Liste der Ausdrücke für Empfängeranmeldungen.
	 */
	private Collection<AnmeldeAusdruck> empfaenger = new ArrayList<>();

	/**
	 * die Liste der Ausdrücke für Quellenanmeldungen.
	 */
	private Collection<AnmeldeAusdruck> quellen = new ArrayList<>();

	/**
	 * die Liste der Ausdrücke für Senderanmeldungen.
	 */
	private Collection<AnmeldeAusdruck> sender = new ArrayList<>();

	/**
	 * die Liste der Ausdrücke für Senkeanmeldungen.
	 */
	private Collection<AnmeldeAusdruck> senken = new ArrayList<>();

	/**
	 * der Name der Variable, in der vom Datenverteiler empfangene Datensätze
	 * bei der Aktualisierung abgelegt werden.
	 */
	private String aktualisierungsVariable;

	/**
	 * die Liste der Anweisungen, die nach dem Empfang von Daten vom
	 * Datenverteiler ausgeführt werden.
	 */
	private AnweisungsBlock aktualisierungsAnweisungen;

	/**
	 * die Liste der Anweisungen, die beim Auftreten unbehandelter Fehler
	 * innerhalb der Anmeldeanweisung ausgeführt werden.
	 */
	private AnweisungsBlock fehlerAnweisungen;

	/**
	 * die Variable in der eine Fehlermeldung abgelegt wird, um sie mit den
	 * Anweisungen aus dem Fehlerbehandlungsbereich auszuwerten.
	 */
	private String fehlerVariable;

	/**
	 * die Liste der Anweisungen, die vor jeden anderen Operation innerhalb der
	 * Anmeldeanweisung einmalig ausgeführt werden.
	 */
	private AnweisungsBlock definitionAnweisungen;

	/**
	 * die Liste der Anweisungen, die nach Abschluss aller Anmeldung einmalig
	 * ausgeführt werden.
	 */
	private AnweisungsBlock initialAnweisungen;

	/**
	 * die Liste der Anweisungen, die nach dem Empfang einer Nachricht über die
	 * Änderung einer dynamsichen Menge ausgeführt werden.
	 */
	private AnweisungsBlock mengenAenderungsAnweidungen;

	/**
	 * der Name der Variable, in der eine dynamische Mengenänderung, die vom
	 * Datenverteiler empfangen wurde abgelegt wird.
	 */
	private String mengenAenderungsVariable;

	/**
	 * Konstruktur.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung deifniert ist.
	 * @param skript
	 *            der Name des Skripts
	 * @param name
	 *            der Name der Variable unter der die Aktivität im umgebenden
	 *            Kontext abgelegt werden soll.
	 */
	public AnmeldeAnweisung(final int zeile, final String skript, final String name) {
		super(zeile, skript);
		this.name = name;
	}

	/**
	 * fügt eine Menge von Anweisungen für den Aktualisierungsberecih ein.
	 *
	 * @param anweisungen
	 *            die Anweisungen
	 */
	public void addAktualisierungsAnweisungen(final AnweisungsBlock anweisungen) {
		aktualisierungsAnweisungen = anweisungen;
	}

	/**
	 * fügt eine Menge von Anweisungen für den Definitionsbereich ein.
	 *
	 * @param anweisungen
	 *            die Anweisungen
	 */
	public void addDefinitionAnweisungen(final AnweisungsBlock anweisungen) {
		definitionAnweisungen = anweisungen;
	}

	/**
	 * fügt eine Menge von Anweisungen für den Fehlerbehandlungsbereich ein.
	 *
	 * @param anweisungen
	 *            die Anweisungen
	 */
	public void addFehlerAnweisungen(final AnweisungsBlock anweisungen) {
		fehlerAnweisungen = anweisungen;
	}

	/**
	 * fügt eine Menge von Anweisungen für den Initialbereich ein.
	 *
	 * @param anweisungen
	 *            die Anweisungen
	 */
	public void addInitialAnweisungen(final AnweisungsBlock anweisungen) {
		initialAnweisungen = anweisungen;
	}

	/**
	 * fügt eine Menge von Anweisungen für den Mengenänderungsbereich ein.
	 *
	 * @param anweisungen
	 *            die Anweisungen
	 */
	public void addMengenAenderungsAnweisungen(final AnweisungsBlock anweisungen) {
		mengenAenderungsAnweidungen = anweisungen;
	}

	/**
	 * liefert den Anweisungsblock des Aktualisierungsbereiches.
	 *
	 * @return die Anweisungen
	 */
	public AnweisungsBlock getAktualisierungsAnweisungen() {
		return aktualisierungsAnweisungen;
	}

	/**
	 * liefert den Namen der Variable in der vom Datenverteiler empfangenen
	 * Daten zur Auswertung im Aktualisierungsbereich abgelegt werden.
	 *
	 * @return den Name
	 * @see #setAktualisierungsVariable(String)
	 */
	public String getAktualisierungsVariable() {
		return aktualisierungsVariable;
	}

	/**
	 * liefert den Anweisungsblock des Definitionsbereiches.
	 *
	 * @return die Anweisungen
	 */
	public AnweisungsBlock getDefinitionAnweisungen() {
		return definitionAnweisungen;
	}

	/**
	 * liefert die Liste der Ausdrücke für die Empfängeranmeldungen.
	 *
	 * @return die Liste
	 * @see #setEmpfaenger(Collection)
	 */
	public Collection<AnmeldeAusdruck> getEmpfaenger() {
		return empfaenger;
	}

	/**
	 * liefert den Anweisungsblock des Fehlerbehandlungsbereiches.
	 *
	 * @return die Anweisungen
	 */
	public AnweisungsBlock getFehlerAnweisungen() {
		return fehlerAnweisungen;
	}

	/**
	 * liefert den Namen der Variable in der ein Fehlerobjekt zur Auswertung im
	 * Fehlerbehandlungsbereich abgelegt wird.
	 *
	 * @return den Name
	 * @see #setFehlerVariable(String)
	 */
	public String getFehlerVariable() {
		return fehlerVariable;
	}

	/**
	 * liefert den Anweisungsblock des Initialbereiches.
	 *
	 * @return die Anweisungen
	 */
	public AnweisungsBlock getInitialAnweisungen() {
		return initialAnweisungen;
	}

	/**
	 * liefert den Anweisungsblock des Mengenänderungsbereiches.
	 *
	 * @return die Anweisungen
	 * @see #setMengenAenderungsVariable(String)
	 */
	public AnweisungsBlock getMengenAenderungsAnweisungen() {
		return mengenAenderungsAnweidungen;
	}

	/**
	 * liefert den Namen der Variable in der vom Datenverteiler empfangenen
	 * Änderungen von dynamischen Mengen zur Auswertung im
	 * Mengenänderungsbereich abgelegt werden.
	 *
	 * @return den Name
	 * @see #setMengenAenderungsVariable(String)
	 */
	public String getMengenAenderungsVariable() {
		return mengenAenderungsVariable;
	}

	/**
	 * liefert den Name der Variable unter dem die Aktivität im umgebenden
	 * Kontext abgelegt wird.
	 *
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * liefert die Liste der Ausdrück zur Anmeldung als Quelle.
	 *
	 * @return die Liste der Ausdrücke
	 * @see #setQuellen(Collection)
	 */
	public Collection<AnmeldeAusdruck> getQuellen() {
		return quellen;
	}

	/**
	 * liefert die Liste der Ausdrück zur Anmeldung als Sender.
	 *
	 * @return die Liste der Ausdrücke
	 * @see #setSender(Collection)
	 */
	public Collection<AnmeldeAusdruck> getSender() {
		return sender;
	}

	/**
	 * liefert die Liste der Ausdrück zur Anmeldung als Senke.
	 *
	 * @return die Liste der Ausdrücke
	 * @see #setSenken(Collection)
	 */
	public Collection<AnmeldeAusdruck> getSenken() {
		return senken;
	}

	/**
	 * Die Funktion erzeugt eine nebenläufige Aktivität innerhalb des
	 * Uda-Interpreters. Eventuell aufgetretende nicht behandelte Fehler bim
	 * Erzeugen der Aktivität werden weitergereicht.
	 *
	 * @param kontext
	 *            der Kontext für die Ausführung
	 * @return die erzeugte Anmeldung
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public final Anmeldung interpret(final Kontext kontext) {
		Anmeldung thread = null;
		try {
			thread = new Anmeldung((SkriptKontext) kontext, name, this);
		} catch (final InterpreterException e) {
			throw erzeugeAnweisungsFehler(e);
		}

		return thread;
	}

	/**
	 * setzt den Namen der Variable in der vom Datenverteiler empfangenen Daten
	 * zur Auswertung im Aktualisierungsbereich abgelegt werden.
	 *
	 * @param text
	 *            der Name
	 * @see #getAktualisierungsVariable()
	 */
	public void setAktualisierungsVariable(final String text) {
		aktualisierungsVariable = text;
	}

	/**
	 * setzt die Liste der Ausdrücke für die Beschreibung von
	 * Empfängeranmeldungen.
	 *
	 * @param liste
	 *            die Liste mit den Ausdrücken
	 * @see #getEmpfaenger()
	 */
	public void setEmpfaenger(final Collection<AnmeldeAusdruck> liste) {
		this.empfaenger = new ArrayList<>(liste);
	}

	/**
	 * setzt den Namen der Variable in der ein Fehlerobjekt zur Auswertung im
	 * Fehlerbehandlungsbereich abgelegt wird.
	 *
	 * @param text
	 *            der Name
	 * @see #getFehlerVariable()
	 */
	public void setFehlerVariable(final String text) {
		fehlerVariable = text;
	}

	/**
	 * setzt den Namen der Variable in der vom Datenverteiler empfangenen
	 * Änderungen von dynamischen Mengen zur Auswertung im
	 * Mengenänderungsbereich abgelegt werden.
	 *
	 * @param text
	 *            der Name
	 * @see #getMengenAenderungsVariable()
	 */
	public void setMengenAenderungsVariable(final String text) {
		mengenAenderungsVariable = text;
	}

	/**
	 * setzt die Liste der Ausdrücke für die Beschreibung von
	 * Quellenanmeldungen.
	 *
	 * @param liste
	 *            die Liste mit den Ausdrücken
	 * @see #getQuellen()
	 */
	public void setQuellen(final Collection<AnmeldeAusdruck> liste) {
		this.quellen = new ArrayList<>(liste);
	}

	/**
	 * setzt die Liste der Ausdrücke für die Beschreibung von Senderanmeldungen.
	 *
	 * @param liste
	 *            die Liste mit den Ausdrücken
	 * @see #getSender()
	 */
	public void setSender(final Collection<AnmeldeAusdruck> liste) {
		this.sender = new ArrayList<>(liste);
	}

	/**
	 * setzt die Liste der Ausdrücke für die Beschreibung von Senkeanmeldungen.
	 *
	 * @param liste
	 *            die Liste mit den Ausdrücken
	 * @see #getSenken()
	 */
	public void setSenken(final Collection<AnmeldeAusdruck> liste) {
		this.senken = new ArrayList<>(liste);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(new Variable(name));
		liste.add(definitionAnweisungen);
		liste.addAll(empfaenger);
		liste.addAll(sender);
		liste.addAll(senken);
		liste.addAll(quellen);
		liste.add(initialAnweisungen);
		liste.add(new Variable(aktualisierungsVariable));
		liste.add(aktualisierungsAnweisungen);
		liste.add(new Variable(fehlerVariable));
		liste.add(fehlerAnweisungen);
		liste.add(new Variable(mengenAenderungsVariable));
		liste.add(mengenAenderungsAnweidungen);
		return liste;
	}
}
