/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung;

import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.dav.DavDaten;
import de.bsvrz.ibv.uda.interpreter.daten.dav.MengenObjekt;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * die Repräsentation einer Mengenänderung.<br>
 * Instanzen dieser Klasse werden erzeugt, wenn eine Änderung für eine
 * dynamische Menge vom Datenverteiler empfangen wurde. Das Objekt wird der
 * Mengenänderungvariable im lokalen Kontext der Anmelde-Aktivität übergeben, so
 * dass die Anweisungen des Mengenänderungsbereiches darauf zugreifen können.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class MengenAenderung implements Struktur {

	/**
	 * markiert, ob der Menge ein Objekt hinzugefügt wurde.
	 */
	private final boolean added;

	/**
	 * das geänderte Mengenobjekt.
	 */
	private final MengenObjekt menge;

	/**
	 * das der Menge hinzugefügte Objekt.
	 */
	private final DavDaten objekt;

	/**
	 * Konstruktor.
	 *
	 * @param menge
	 *            die geänderte Menge
	 * @param objekt
	 *            das hinzugefügte oder entfernte Objekt
	 * @param added
	 *            Objekt wurde hinzugefügt ?
	 */
	public MengenAenderung(final MengenObjekt menge, final DavDaten objekt, final boolean added) {
		this.menge = menge;
		this.objekt = objekt;
		this.added = added;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Folgende Attribute können ermittelt werden:
	 * <ul>
	 * <li><b>entfernt:</b> liefert "wahr", wenn das enthaltene Objekt aus der
	 * Menge entfernt wurde.</li>
	 * <li><b>erweitert:</b> liefert "wahr", wenn das enthaltene Objekt in der
	 * Menge hinzugefügt wurde.</li>
	 * <li><b>objekt:</b> liefert das hinzugefügte oder entfernte Objekt.</li>
	 * <li><b>mengenObjekt:</b> liefert das Datenverteilerobjekt, das die Menge
	 * repräsentiert.</li>
	 * </ul>
	 *
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object result = null;
		if ("entfernt".equals(name)) {
			result = LogischerWert.valueOf(!added);
		} else if ("erweitert".equals(name)) {
			result = LogischerWert.valueOf(added);
		} else if ("objekt".equals(name)) {
			result = objekt;
		} else if ("mengenObjekt".equals(name)) {
			result = menge;
		}

		if (result == null) {
			throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT);
		}
		return result;
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Attribute einer Mengenänderung können von außen nicht gesetzt werden.
	 * Der Aufruf der Funktion liefert daher in jedem Fall einen
	 * Objektzugriffsfehler.
	 *
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.OBJEKT);
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer();
		buffer.append(menge.toString());
		buffer.append(':');
		buffer.append(objekt.toString());
		buffer.append(':');
		if (added) {
			buffer.append("hinzugefügt");
		} else {
			buffer.append("entfernt");
		}
		return buffer.toString();
	}
}
