/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AnweisungsBlock;
import de.bsvrz.ibv.uda.interpreter.anweisung.AbstractAnweisung;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.ZeichenkettenLiteral;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * Repräsentation eines Ausdrucks um Definieren einer Funktion innerhalb eines
 * Uda-Skripts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class FunktionsDefinition extends AbstractAnweisung {
	/**
	 * der Name der Funktion.
	 */
	private final String name;

	/**
	 * die Liste der Namen der Argumente der Funktion.
	 */
	private final List<String> argumente = new ArrayList<>();

	/**
	 * der Anweisungsblock, der die Funktionalität der Funktion enthält.
	 */
	private final AnweisungsBlock anweisungen;

	/**
	 * Konstanten, die von der Funktion vorrangig vor Einträgen im Kontext
	 * verwendet werden.
	 */
	private final Kontext konstanten = new Kontext();

	/**
	 * Konstruktor.<br>
	 *
	 * @param zeile
	 *            die Nummer der Zeile im Quelltext des Skripts
	 * @param skript
	 *            der Name des Skripts
	 * @param name
	 *            der Name der Funktion
	 * @param argumente
	 *            die Argumente der Funktion
	 * @param anweisungen
	 *            die Anweisungen innerhalb der Funktion
	 */
	public FunktionsDefinition(final int zeile, final String skript, final String name, final List<String> argumente,
			final AnweisungsBlock anweisungen) {
		super(zeile, skript);
		this.name = name;
		if (argumente != null) {
			this.argumente.addAll(argumente);
		}
		this.anweisungen = anweisungen;
	}

	/**
	 * liefert den Anweisungsblock der Funktion.
	 *
	 * @return der Anweisungsblock
	 */
	public AnweisungsBlock getAnweisungen() {
		return anweisungen;
	}

	/**
	 * liefert die Liste der Argumentnamen der Funktion.
	 *
	 * @return die Liste
	 */
	public List<String> getArgumente() {
		return argumente;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		for (final String argName : argumente) {
			liste.add(new ZeichenkettenLiteral(argName));
		}
		liste.add(anweisungen);
		return liste;
	}

	/**
	 * liefert den Name der Funktion.
	 *
	 * @return den Name
	 */
	public String getName() {
		return name;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		for (final String varName : konstanten.getVariablen()) {
			kontext.set(varName, konstanten.get(varName));
		}
		return anweisungen.interpret(kontext);
	}

	/**
	 * setzt eine Konstante, die zum Auswerten der Funktion verwendet wird.
	 *
	 * @param varName
	 *            der Name der Konstante
	 * @param objekt
	 *            der Wert
	 */
	public void setKonstante(final String varName, final Object objekt) {
		konstanten.set(varName, objekt);
	}
}
