/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.container.UdaContainer;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.Fehler;
import de.bsvrz.ibv.uda.interpreter.daten.konstante.ZeichenkettenLiteral;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Repräsentation einer Quantität innerhalb eines Quantors.<br>
 * Die Quantorbedingung prüft über einer Menge von boolschen Ergebnissen, ob
 * eine geforderte Quantität an prüfungsergebnissen erreicht wurde.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class QuantorBedingung implements Ausdruck {

	/**
	 * Konstante für: die Bedingung ist nicht definiert.
	 */
	public static final int UNDEFINED = 0;

	/**
	 * Konstante für: die übergebene Ergebnismenge enthält nur den Wert WAHR.
	 */
	public static final int ALLE = 1;

	/**
	 * Konstante für: die Ergebnismenge enthält mindestens ein Element mit dem
	 * Wert WAHR.
	 */
	public static final int EIN = 2;

	/**
	 * Konstante für: die Ergebnismenge enthält eine Mindestanzahl an Elementen
	 * mit dem Wert WAHR.
	 */
	public static final int MINDESTENS = 3;

	/**
	 * Konstante für: die Ergebnismenge enthält nur eine Höchstanzahl von
	 * Elementen mit dem Wert WAHR.
	 */
	public static final int HOECHSTENS = 4;

	/**
	 * Konstante für: die Ergebnismenge wird von einer nutzerdefnierten Funktion
	 * ausgewertet.
	 */
	public static final int NUTZER = 5;

	/**
	 * der Typ der Quantorbedingung.
	 */
	private int typ = QuantorBedingung.UNDEFINED;

	/**
	 * der Name der Quantorbedingung (wird für nutzerdefinierte Bedingungen
	 * benötigt).
	 */
	private final String name;

	/**
	 * der Größenwert für anzahlabhängige Bedingungen.
	 */
	private final Ausdruck groesse;

	/**
	 * die Größenangabe erfolgt in Prozent ?
	 */
	private final boolean prozent;

	/**
	 * Konstruktor.
	 *
	 * @param typ
	 *            der Typ der Quantorbedingung
	 * @param name
	 *            der Name der Bedingung
	 * @param groesse
	 *            der Ausdruck für eine Größenangabe
	 * @param prozent
	 *            Größenangabe erfolgt in Prozent ?
	 */
	public QuantorBedingung(final int typ, final String name, final Ausdruck groesse, final boolean prozent) {
		this.typ = typ;
		this.name = name;
		this.groesse = groesse;
		this.prozent = prozent;
	}

	/**
	 * überprüft die Gültigkeit der Quantorbedingung.<br>
	 *
	 * @param ergebnisse
	 *            die Liste der Prüfungsergebnisse der Elemente der Menge
	 * @param wahr
	 *            die Anzahl der WAHR-Ergebnisse
	 * @param falsch
	 *            die Anzahl der FALSCH-Ergebnisse
	 * @param kontext
	 *            der Kontext zur Auswertung der Bedingung
	 * @return das Testergebnis
	 */
	public LogischerWert check(final UdaContainer ergebnisse, final int wahr, final int falsch,
			final SkriptKontext kontext) {
		LogischerWert result = LogischerWert.FALSCH;
		Number groessenWert = null;
		switch (typ) {
		case ALLE:
			if (falsch == 0) {
				result = LogischerWert.WAHR;
			}
			break;
		case EIN:
			if (wahr > 0) {
				result = LogischerWert.WAHR;
			}
			break;
		case MINDESTENS:
			groessenWert = (Number) interpret(kontext);
			if (prozent) {
				if (ergebnisse.getGroesse() > 0) {
					final double prWert = ((double) wahr / (double) ergebnisse.getGroesse()) * 100.0;
					if (prWert >= groessenWert.doubleValue()) {
						result = LogischerWert.WAHR;
					}
				} else {
					result = LogischerWert.WAHR;
				}
			} else {
				if (wahr >= groessenWert.intValue()) {
					result = LogischerWert.WAHR;
				}
			}
			break;
		case HOECHSTENS:
			groessenWert = (Number) interpret(kontext);
			if (prozent) {
				if (ergebnisse.getGroesse() > 0) {
					final double prWert = ((double) wahr / (double) ergebnisse.getGroesse()) * 100.0;
					if (prWert <= groessenWert.doubleValue()) {
						result = LogischerWert.WAHR;
					}
				} else {
					result = LogischerWert.WAHR;
				}
			} else {
				if (wahr <= groessenWert.intValue()) {
					result = LogischerWert.WAHR;
				}
			}
			break;
		case NUTZER:
			final QuantorDefinition qDef = kontext.getInterpreter().getQuantor(name);
			if (qDef == null) {
				throw new Fehler("Keine Quantordefinition mit dem Namen \""
						+ name + "\" gefunden");
			}
			final SkriptKontext quantorKontext = new SkriptKontext(kontext, true);
			quantorKontext.set(qDef.getMengenName(), ergebnisse);
			final Ausdruck ausdruck = (Ausdruck) qDef.interpret(quantorKontext);
			final Object auswertungsErgebnis = ausdruck.interpret(quantorKontext);
			if (auswertungsErgebnis instanceof LogischerWert) {
				result = (LogischerWert) auswertungsErgebnis;
			} else {
				throw new Fehler("Die Auswertung des Quantors liefert keinen logischen Wert");
			}
			kontext.reset(qDef.getMengenName(), quantorKontext);
			break;
		default:
			throw new UnsupportedOperationException("Der Quantortyp " + typ
					+ " wird nicht unterstützt");
		}

		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		if (groesse != null) {
			liste.add(groesse);
		} else if (name != null) {
			liste.add(new ZeichenkettenLiteral(name));
		}
		return liste;
	}

	/**
	 * liefert den Name der Bedingung.
	 *
	 * @return den Namen
	 */
	String getName() {
		return name;
	}

	/**
	 * liefert den Typ der Bedingung.
	 *
	 * @return den Typ
	 */
	int getTyp() {
		return typ;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = null;
		if (groesse != null) {
			result = groesse.interpret(kontext);
		}
		return result;
	}
}
