/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.ausdruck;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;
import de.bsvrz.ibv.uda.parser.UDATokensTokenTypes;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * Die Klasse beschreibt spezielle durch ein Schlüsselwort beschriebene
 * Ausdrücke in einem Uda-Skript.<br>
 * Definiert sind die Namen <b>undefiniert</b> und <b>aktuell</b>. Der Ausdruck
 * <b>undefiniert</b> liefert das konstante Objekt für einen "Nichtwert", der
 * Ausdruck <b>aktuell</b> liefert einen Zeitstempel, der den aktuellen
 * Zeitpunkt beschreibt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaBasisAusdruck implements Ausdruck {

	/**
	 * die Liste der innerhalb Uda definierten Sonderausdrücke.
	 */
	private static final int[] AUSDRUECKE = { UDATokensTokenTypes.Aktuell, UDATokensTokenTypes.Undefiniert };

	/**
	 * der Typ des Ausdrucks.
	 */
	private final int ausdrucksTyp;

	/**
	 * Konstruktor für den Ausdruck mit dem übergebenen Namen.<br>
	 * Wird ein nicht definierter Name übergeben, wird eine
	 * <i>UnsupportedOperationException</i> geworfen.
	 *
	 * @param typ
	 *            der Typ des Ausdrucks
	 */
	public UdaBasisAusdruck(final int typ) {
		if (!isDefiniert(typ)) {
			throw new UnsupportedOperationException("Der Ausdruck wird nicht unterstützt");
		}
		this.ausdrucksTyp = typ;
	}

	/**
	 * die Funktion prüft, ob der übergebene Typ für einen Sonderausdruck
	 * unterstützt wird.
	 *
	 * @param typ
	 *            der Typ
	 * @return "wahr", wenn der Typ unterstützt wird.
	 */
	private boolean isDefiniert(final int typ) {
		boolean ergebnis = false;
		for (final Integer eintrag : UdaBasisAusdruck.AUSDRUECKE) {
			if (eintrag == typ) {
				ergebnis = true;
				break;
			}
		}
		return ergebnis;
	}

	/**
	 * Die Funktion wertet den Ausdruck aus und liefert entsprechend des Typs
	 * ein Objekt zur weiteren Verarbeitung innerhalb des Skriptinterpreters.
	 * <br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object ergebnisObjekt = null;
		switch (ausdrucksTyp) {
		case UDATokensTokenTypes.Undefiniert:
			ergebnisObjekt = NichtWert.NICHTWERT;
			break;
		case UDATokensTokenTypes.Aktuell:
			ergebnisObjekt = new Zeitstempel(null);
			break;
		default:
			throw new UnsupportedOperationException();
		}
		return ergebnisObjekt;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		return null;
	}

}
