/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;

/**
 * ein Ausdruck, der den Zugriff auf ein Element eines Container über einen
 * Index beschreibt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ContainerZugriffsSymbol implements Ausdruck {

	/**
	 * der Ausdruck, der das Containerobjekt beschreibt, aus dem ein Element
	 * ausgewählt werden soll.
	 */
	private final Ausdruck objektAusdruck;

	/**
	 * der Ausdruck, der den Index beschreibt, mit dem der Zugraiff erfolgen
	 * soll.
	 */
	private final Ausdruck indexAusdruck;

	/**
	 * Konstruktor.<br>
	 *
	 * @param objektAusdruck
	 *            Ausdruck der das Objekt (einen Container) beschreibt
	 * @param indexAusdruck
	 *            Ausdruck, der den Index innerhalb des Containers beschreibt
	 */
	public ContainerZugriffsSymbol(final Ausdruck objektAusdruck, final Ausdruck indexAusdruck) {
		this.objektAusdruck = objektAusdruck;
		this.indexAusdruck = indexAusdruck;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = null;
		final Object object = objektAusdruck.interpret(kontext);
		final Object indexObject = indexAusdruck.interpret(kontext);
		if (object instanceof Container) {
			if (indexObject instanceof Long) {
				final int index = ((Number) indexObject).intValue();
				result = ((Container) object).getContainerElement(index);
			} else if (indexObject instanceof SelektionsBedingung) {
				if (object instanceof UdaContainer) {
					final SelektionsBedingung selektion = (SelektionsBedingung) indexObject;
					result = selektion.select((UdaContainer) object, true);
				}
			} else if (object instanceof SchluesselMenge) {
				final SchluesselMenge sm = (SchluesselMenge) object;
				result = sm.getContainerElement(indexObject.toString());
			}
		}

		return result;
	}

	/**
	 * setzt die Daten eines Objekts.<br>
	 * Die Funktion setzt die Daten des Objekts, das durch den
	 * {@link #objektAusdruck} und den {@link #indexAusdruck} beschrieben ist
	 * auf den übergebenen Wert.
	 *
	 * @param kontext
	 *            der Kontext für die Ausführung der Operation
	 * @param daten
	 *            die zu setzenden Daten
	 */
	public void set(final Kontext kontext, final Object daten) {
		final Object object = objektAusdruck.interpret(kontext);
		final Object indexObject = indexAusdruck.interpret(kontext);
		if (object instanceof SchluesselMenge) {
			final SchluesselMenge sm = (SchluesselMenge) object;
			sm.setContainerElement(indexObject.toString(), daten);
		} else if (indexObject instanceof Long) {
			if (object instanceof Container) {
				final int index = ((Number) indexObject).intValue();
				((Container) object).setContainerElement(index, daten);
			}
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#getNachfolger()
	 */
	@Override
	public List<Ausdruck> getNachfolger() {
		final List<Ausdruck> liste = new ArrayList<>();
		liste.add(objektAusdruck);
		liste.add(indexAusdruck);
		return liste;
	}

}
