/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.dav;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataAndATGUsageInformation;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.KonfigurationsKontext;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.KonfigurationsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.dynobj.DynObjektException;
import de.bsvrz.sys.funclib.dynobj.DynamischeObjekte;

/**
 * Handler zur Ausführung von Operationen mit der Datenverteiler-Konfiguration.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class KonfigurationsHandler extends AbstractUdaHandler {
	/**
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return KonfigurationsOperatoren.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {
		erzeugeHandlerFehler(operator, operanden);

		final KonfigurationsKontext kontext = (KonfigurationsKontext) operanden.get(0);

		final ClientDavInterface connection = kontext.getInterpreter().getConnection();

		final DataModel model = connection.getDataModel();
		String pid = null;
		if (operanden.get(1) instanceof String) {
			pid = (String) operanden.get(1);
		} else if (operanden.get(1) instanceof TypObjekt) {
			pid = ((SystemObjectType) ((TypObjekt) operanden.get(1)).getObjekt()).getPid();
		} else if (operanden.get(1) instanceof AtgObjekt) {
			pid = ((AtgObjekt) operanden.get(1)).getObjekt().getPid();
		}
		Object result = null;

		if (pid != null) {
			if (KonfigurationsOperatoren.ASPEKT_ERMITTELN.equals(operator)) {
				final Aspect asp = model.getAspect(pid);
				if (asp != null) {
					result = DavDaten.erzeugeObjekt(connection, asp);
				}
			} else if (KonfigurationsOperatoren.ATG_ERMITTELN.equals(operator)) {
				final AttributeGroup atg = model.getAttributeGroup(pid);
				if (atg != null) {
					result = DavDaten.erzeugeObjekt(connection, atg);
				}
			} else if (KonfigurationsOperatoren.OBJEKT_ERMITTELN.equals(operator)) {
				final SystemObject obj = model.getObject(pid);
				if (obj != null) {
					result = DavDaten.erzeugeObjekt(connection, obj);
				}
			} else if (KonfigurationsOperatoren.TYP_ERMITTELN.equals(operator)) {
				final SystemObjectType typ = model.getType(pid);
				if (typ != null) {
					result = DavDaten.erzeugeObjekt(connection, typ);
				}
			} else if (KonfigurationsOperatoren.NEUES_OBJEKT.equals(operator)) {
				final SystemObjectType typ = model.getType(pid);
				if (typ == null) {
					throw new KonfigurationsFehler(UdaFehlerSubtyp.OBJEKT,
							"Für die PID: \"" + pid + "\" existiert kein SystemObjectTyp");
				} else if (typ instanceof DynamicObjectType) {
					String neuePid = "";
					try {
						String name = "";
						if (operanden.size() > 2) {
							neuePid = (String) operanden.get(2);
						}
						if (operanden.size() > 3) {
							name = (String) operanden.get(3);
						}

						// final ConfigurationArea area =
						// connection.getLocalConfigurationAuthority()
						// .getConfigurationArea();
						final Collection<DataAndATGUsageInformation> konfigData = new ArrayList<>();

						for (int index = 4; index < operanden.size(); index++) {
							final Object op = operanden.get(index);
							if (op instanceof DataAndAtgUsageInformationObjekt) {
								konfigData.add(((DataAndAtgUsageInformationObjekt) op).getDavInformation());
							}
						}

						result = DavDaten.erzeugeObjekt(connection, DynamischeObjekte.getInstanz(connection)
								.erzeugeObjekt((DynamicObjectType) typ, neuePid, name, konfigData));
					} catch (final DynObjektException e) {
						UdaFehlerSubtyp subtyp = UdaFehlerSubtyp.UNBEKANNT;
						if ((neuePid != null) && (neuePid.length() > 0)) {
							if (connection.getDataModel().getObject(neuePid) != null) {
								subtyp = UdaFehlerSubtyp.PID;
							}
						}
						throw new KonfigurationsFehler(subtyp, e.getMessage());
					}
				} else {
					throw new KonfigurationsFehler(UdaFehlerSubtyp.OBJEKT,
							"Der Typ des anzulegenden Objekts: " + pid + " beschreibt kein dynamisches Objekt");
				}
			} else if (KonfigurationsOperatoren.DATA_AND_ATG_USAGE_INFORMATION.equals(operator)) {
				AtgObjekt atgObj = null;
				AspektObjekt aspObj = null;

				Object operand = operanden.get(1);
				if (operand instanceof String) {
					final AttributeGroup atg = model.getAttributeGroup((String) operand);
					if (atg == null) {
						throw new KonfigurationsFehler(UdaFehlerSubtyp.OBJEKT,
								"Die PID: " + operand + " beschreibt keine Attributgruppe");
					}
					atgObj = (AtgObjekt) DavDaten.erzeugeObjekt(connection, atg);
				} else if (operand instanceof AtgObjekt) {
					atgObj = (AtgObjekt) operand;
				}

				operand = operanden.get(2);
				if (operand instanceof String) {
					final Aspect asp = model.getAspect((String) operand);
					if (asp == null) {
						throw new KonfigurationsFehler(UdaFehlerSubtyp.OBJEKT,
								"Die PID: " + operand + " beschreibt keinen Aspekt");
					}
					aspObj = (AspektObjekt) DavDaten.erzeugeObjekt(connection, asp);
				} else if (operand instanceof AspektObjekt) {
					aspObj = (AspektObjekt) operand;
				}

				result = new DataAndAtgUsageInformationObjekt(atgObj, aspObj, (DavDatenObjekt) operanden.get(3));
			}
			if (result == null) {
				// throw new ObjektZugriffsFehler(UdaFehlerSubtyp.OBJEKT,
				// "Kein Objekt mit der PID " + pid + " gefunden");
				//
				result = new ObjektZugriffsFehler(UdaFehlerSubtyp.OBJEKT,
						"Kein Objekt mit der PID " + pid + " gefunden");
			}
		}
		return result;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOk = false;
		boolean typOk = false;
		if (operanden.size() > 1) {
			anzahlOk = true;
			if (operanden.get(0) instanceof KonfigurationsKontext) {
				if (operanden.get(1) instanceof String) {
					typOk = true;
				} else {
					if (KonfigurationsOperatoren.NEUES_OBJEKT.equals(operator)) {
						if (operanden.get(1) instanceof TypObjekt) {
							typOk = true;
						}
					} else if (KonfigurationsOperatoren.DATA_AND_ATG_USAGE_INFORMATION.equals(operator)) {
						anzahlOk = false;
						typOk = false;
						if (operanden.size() == 4) {
							anzahlOk = true;
							if ((operanden.get(1) instanceof AtgObjekt) || (operanden.get(1) instanceof String)) {
								if ((operanden.get(2) instanceof AspektObjekt)
										|| (operanden.get(2) instanceof String)) {
									if (operanden.get(3) instanceof DavDatenObjekt) {
										typOk = true;
									}
								}
							}
						}
					}

				}
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}
}
