/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.fehler;

import com.bitctrl.Constants;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;

/**
 * Abstrakte Definition eines Fehler innerhalb des Uda-Skripts-Interpreters.<br>
 * UdaFehler sind Fehler, die durch Probleme in der Programmierung eines
 * Uda-Skripts verursacht werden. Bei der Abarbeitung innerhalb eines Skripts
 * werden diese Fehler-Exceptions durch die entsprechende Auswertungsfunktion
 * einer Anweisung abgefangen und entsprechend der gültigen
 * Fehlerbahnadlungsstrategie weitergereicht und ausgwertet.<br>
 * Die Klasse implementiert die Schnittstelle Struktur, um den Zugriff auf ihre
 * Elemente über Namen zu ermöglichen. Folgende Elemente können abgefragt
 * werden:
 * <ol>
 * <li>typ - der Typ des Fehlers als String</li>
 * <li>subtyp - der Subtyp des Fehlers als String</li>
 * <li>subtypText - der Text des Subtyp des Fehlers als String</li>
 * <li>zusatztext - die Meldung die dem Fehler zugeordnet ist</li>
 * <li>funktion - die Funktion in der der Fehler aufgetreten ist, falls er in
 * einer Funktion aufgetreten ist</li>
 * <li>skript - die Bezeichnung des Skripts, in dem der Fehler aufgetreten ist
 * </li>
 * <li>zeile - die Zeile des Quelltextes in der die den Fehler verurschende
 * Anweisung zu finden ist</li>
 * <li>ursprünglicherFehler - der Fehler, der den Fehler ausgelöst hat, wenn es
 * sich um einen Folgefehlerhandelt</li>
 * </ol>
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class UdaFehler extends InterpreterException implements Struktur, Kopierbar {

	/**
	 * Versions-ID.
	 */
	static final long serialVersionUID = 1L;

	/**
	 * der Meldungstext des Fehlers.
	 */
	private String zusatztext = "";

	/**
	 * der ursprüngliche Fehler.
	 */
	private UdaFehler vorgaenger;

	/**
	 * die Zeile innerhalb des Quelltextes, in der der Fehler aufgetreten ist.
	 */
	private int zeile = -1;

	/**
	 * der Name des Skripts, in dem der Fehler aufgetreten ist.
	 */
	private String skript = "";

	/**
	 * der Subtyp des Fehlers.
	 */
	private final UdaFehlerSubtyp subTyp;

	/**
	 * der Name der Funktion, in der ein Fehler aufgetreten ist.
	 */
	private String funktion;

	/**
	 * Standardkonstruktor.
	 */
	public UdaFehler() {
		this(UdaFehlerSubtyp.UNBEKANNT, "", null);
	}

	/**
	 * Konstruktor, der die Daten eines übergebenen Fehlerobjekts verwendet.
	 *
	 * @param fehler
	 *            der ursprüngliche Fehler
	 */
	public UdaFehler(final UdaFehler fehler) {
		if (fehler == null) {
			throw new IllegalArgumentException("Das übergebene Fehlerobjekt darf nicht \"null\" sein");
		}
		funktion = fehler.getFunktion();
		skript = fehler.getSkriptName();
		subTyp = fehler.getSubTyp();
		vorgaenger = fehler.getVorgaenger();
		zeile = fehler.getZeile();
		zusatztext = fehler.getZusatztext();
	}

	/**
	 * Konstruktor mit Angabe des Untertypen.
	 *
	 * @param subtyp
	 *            der Subtyp des Fehlers
	 */
	public UdaFehler(final UdaFehlerSubtyp subtyp) {
		this(subtyp, "", null);
	}

	/**
	 * Konstruktor mit Angabe der Fehlermeldung.
	 *
	 * @param subtyp
	 *            der Subtyp des Fehlers
	 * @param text
	 *            die Meldung
	 */
	public UdaFehler(final UdaFehlerSubtyp subtyp, final String text) {
		this(subtyp, text, null);
	}

	/**
	 * Konstruktor mit Angabe von Meldungstext und Vorgängerfehler.
	 *
	 * @param subtyp
	 *            der Subtyp des Fehlers
	 * @param text
	 *            die Meldung des Fehlers
	 * @param vorgaenger
	 *            der ursprüngliche Fehler
	 */
	public UdaFehler(final UdaFehlerSubtyp subtyp, final String text, final UdaFehler vorgaenger) {
		this.subTyp = subtyp;
		setMeldung(text);
		setVorgaenger(vorgaenger);
	}

	/**
	 * liefert den Fehler, der diesen Fehler ausgelöst hat, sofern es sich um
	 * einen Folgefehler handelt. Anderenfalls wird <i>null</i> geliefert.
	 *
	 * @return den Fehler oder <i>null</i>
	 */
	public UdaFehler getFehler() {
		return vorgaenger;
	}

	/**
	 * liefert den Typ des Fehlers als Zeichenkette. Die Zeichenkette entspricht
	 * dem Name der Fehlerklasse, mit dem der Fehler innerhalb eines
	 * Uda-Skriptes erzeugt werden kann.
	 *
	 * @return den Name des Typs
	 */
	public String getFehlerTyp() {
		return toString();
	}

	/**
	 * liefert den Namen der Funktion, in der der Fehler aufgetreten ist, sofern
	 * es sich um eine Funktion handelt. Anderenfalls wird <i>null</i>
	 * geliefert.
	 *
	 * @return den Name der Funktion oder <i>null</i>
	 * @see #setFunktion(String)
	 */
	public String getFunktion() {
		if (funktion == null) {
			funktion = Constants.EMPTY_STRING;
		}
		return funktion;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Throwable#getMessage()
	 */
	@Override
	public String getMessage() {
		final StringBuffer result = new StringBuffer();
		if (super.getMessage() != null) {
			result.append(super.getMessage());
		}

		UdaFehler fehler = this;
		while (fehler != null) {
			if (result.length() > 0) {
				result.append(Constants.NL);
			}

			result.append(getMessageText());
			fehler = fehler.getVorgaenger();
		}

		return result.toString();
	}

	/**
	 * liefert den Meldungstext in der geforderten Form für die Funktion
	 * "meldung".
	 *
	 * @return den Text
	 */
	private String getMessageText() {
		final StringBuffer result = new StringBuffer();
		result.append("typ = ");
		result.append(getClass().getSimpleName());
		result.append(" zusatztext = \"");
		result.append(zusatztext);
		result.append("\" funktion = ");
		result.append(getFunktion());
		result.append(" skript = ");
		result.append(getSkriptName());
		result.append(" zeile = ");
		result.append(zeile);

		return result.toString();
	}

	/**
	 * liefert den Name des Skripts, in dem der Fehler aufgetreten ist. Ist der
	 * Name nicht definiert, wird ein Leerstring geliefert.
	 *
	 * @return den Name des Skripts
	 */
	public String getSkriptName() {
		return skript;
	}

	/**
	 * liefert das über den übergebenen Namen identifizierte Element aus dem
	 * Fehlerobjekt zurüück.<br>
	 * Folgende Elemente können abgefragt werden.
	 * <ol>
	 * <li>typ - der Typ des Fehlers als String</li>
	 * <li>zusatztext - die Meldung die dem Fehler zugeordnet ist</li>
	 * <li>funktion - die Funktion in der der Fehler aufgetreten ist, falls er
	 * in einer Funktion aufgetreten ist</li>
	 * <li>skript - die Bezeichnung des Skripts, in dem der Fehler aufgetreten
	 * ist</li>
	 * <li>zeile - die Zeile des Quelltextes in der die den Fehler verurschende
	 * Anweisung zu finden ist</li>
	 * <li>ursprünglicherFehler - der Fehler, der den Fehler ausgelöst hat, wenn
	 * es sich um einen Folgefehlerhandelt</li>
	 * </ol>
	 *
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		Object ergebnis = null;
		if ("typ".equals(name)) {
			ergebnis = getFehlerTyp();
		} else if ("subtyp".equals(name)) {
			ergebnis = subTyp.name();
		} else if ("subtypText".equals(name)) {
			ergebnis = subTyp.toString();
		} else if ("zusatztext".equals(name)) {
			ergebnis = getZusatztext();
		} else if ("funktionsName".equals(name)) {
			ergebnis = getFunktion();
		} else if ("skript".equals(name)) {
			ergebnis = getSkriptName();
		} else if ("zeile".equals(name)) {
			ergebnis = Long.valueOf(getZeile());
		} else if ("ursprünglicherFehler".equals(name)) {
			ergebnis = getVorgaenger();
		}
		return ergebnis;
	}

	/**
	 * liefert den Subtyp des Fehlers.
	 *
	 * @return subTyp
	 */
	public UdaFehlerSubtyp getSubTyp() {
		return subTyp;
	}

	/**
	 * liefert den Fehler, der diesen verursacht hat, wenn es sich um einen
	 * Folgefehler handelt.
	 *
	 * @return vorgaenger den vorigen Fehler
	 * @see #setVorgaenger(UdaFehler)
	 */
	public UdaFehler getVorgaenger() {
		return vorgaenger;
	}

	/**
	 * liefert die Zeile, in der der Fehler verursacht wurde. Ist die Zeile
	 * nicht bekannt, wird der Wert -1 geliefert.
	 *
	 * @return die Zeile oder -1.
	 * @see #setZeile(int)
	 */
	public int getZeile() {
		return zeile;
	}

	/**
	 * liefert den Meldungstext, mit dem der Fehler erzeugt wurde. Wurde kein
	 * spezieller Meldungstext übergeben, ist das Ergebnis der Funktion ein
	 * Leerstring.
	 *
	 * @return die Meldung
	 * @see #setMeldung(String)
	 */
	public String getZusatztext() {
		return zusatztext;
	}

	/**
	 * liefert die Meldungen des Fehlers und die aller Fehler, die diesen
	 * verursacht haben bis zur angegebenen Suchtiefe.
	 *
	 * @param tiefe
	 *            die Suchtiefe
	 * @return der verkettete Meldungstext
	 */
	public String meldungen(final int tiefe) {
		final StringBuffer buffer = new StringBuffer();
		int level = 0;
		int offeneKlammern = 0;
		UdaFehler fehler = this;

		while ((fehler != null) && ((tiefe == 0) || (level < tiefe))) {
			if (buffer.length() > 0) {
				buffer.append(" ursprünglicherFehler = (");
				offeneKlammern++;
			}
			buffer.append(fehler.getMessageText());
			fehler = fehler.getVorgaenger();
			level++;
		}

		while (offeneKlammern > 0) {
			buffer.append(")");
			offeneKlammern--;
		}

		return buffer.toString();
	}

	/**
	 * setzt den Namen der Funktion, in der der Fehler aufgetreten ist, sofern
	 * es sich um eine Funktion handelt.
	 *
	 * @param name
	 *            der Name der Funktion
	 * @see #getFunktion()
	 */
	public void setFunktion(final String name) {
		funktion = name;
		UdaFehler grund = getVorgaenger();
		while ((grund != null) && (grund.getFunktion().length() <= 0)) {
			grund.setFunktion(name);
			grund = grund.getVorgaenger();
		}
	}

	/**
	 * setzt die Meldung des Fehlerobjekts.
	 *
	 * @param meldung
	 *            festzulegende Meldung
	 * @see #getZusatztext()
	 */
	public void setMeldung(final String meldung) {
		if (meldung != null) {
			this.zusatztext = meldung;
		} else {
			this.zusatztext = "";
		}
	}

	/**
	 * setzt den Name des Skripts, in dem der Fehler aufgetreten ist.
	 *
	 * @param skript
	 *            der Name
	 */
	public void setSkript(final String skript) {
		this.skript = skript;

	}

	/**
	 * die Funktion implementiert die Schnittstelle zum setzen des Wertes eines
	 * Strukturelements. Fehlerobjekte können über das Skript nicht geändert
	 * werden, deshalb wird beim Aufruf der Funktion in jedem Fall ein
	 * Obektzugriffsfehler erzeugt. {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.METHODE,
				"Der Inhalt von Fehlerobjekten kann nicht geändert werden");
	}

	/**
	 * setzt den Fehler, der diesen verursacht.
	 *
	 * @param vorgaenger
	 *            festzulegender Vorgänger
	 * @see #getVorgaenger()
	 */
	public void setVorgaenger(final UdaFehler vorgaenger) {
		this.vorgaenger = vorgaenger;
	}

	/**
	 * setzt die Zeile des Quelltextes des Skripts, in dem der Fehler
	 * aufgetreten ist.
	 *
	 * @param zeile
	 *            Festzulegender zeile
	 * @see #getZeile()
	 */
	public void setZeile(final int zeile) {
		this.zeile = zeile;
	}

	/**
	 * liefert eine Repräsentation des Fehlers als Zeichenkette (entspricht dem
	 * einfachen Klassenname). {@inheritDoc}.
	 *
	 * @see java.lang.Throwable#toString()
	 */
	@Override
	public String toString() {
		return getClass().getSimpleName();
	}
}
