/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.struktur;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;
import de.bsvrz.ibv.uda.interpreter.UdaInterpreter;
import de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur;
import de.bsvrz.ibv.uda.interpreter.daten.container.Container;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehlerSubtyp;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * Die Klasse beschreibt eine Instanz einer benutzerdefinierten Datenstruktur.
 * <br>
 * Sie implementiert die Schnittstellen Container und Struktur, um den Zugriff
 * auf ihre Elemente per Index oder per Name zu ermöglichen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DatenStruktur implements Struktur, Container, Kopierbar {

	/**
	 * die Daten mit den zugeordneten Attributnamen.
	 */
	private final Map<String, Object> fields = new HashMap<>();

	/**
	 * die Datenstrukturdefinition aus der die Instanz hervorgegangen ist.
	 */
	private final DatenStrukturDefinition definition;

	/**
	 * Konstruktor mit Angabe der zugeordneten Definition und den Initialwerten
	 * der Attribute.<br>
	 * Die Angabe der Initialwerte ist optional. Die Zuordnung der Daten aus der
	 * übergebenen Werteliste erfolgt entsprechend der definierten Reihenfolge
	 * der Attribute in der Datenstrukturdefinition.<br>
	 * Alle nicht belegten Attribute werden mit dem "Nichtwert" initialisiert.
	 *
	 * @param definition
	 *            die Datenstrukturdefinition aus der eine Instanz gebildet
	 *            werden soll.
	 * @param daten
	 *            die Intialdaten für die Attribute der Struktur
	 */
	public DatenStruktur(final DatenStrukturDefinition definition, final Object[] daten) {
		if ((daten != null) && (daten.length > definition.getFields().length)) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ANZAHL);
		}
		this.definition = definition;
		int idx = 0;
		for (final String field : definition.getFields()) {
			Object dataElement = null;
			if ((daten != null) && (idx < daten.length)) {
				dataElement = daten[idx++];
			} else {
				dataElement = NichtWert.NICHTWERT;
			}
			this.fields.put(field, dataElement);
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public Kopierbar erzeugeKopie() {
		final DatenStruktur result = new DatenStruktur(definition, null);
		for (final String field : definition.getFields()) {
			result.setStrukturElement(field, getStrukturElement(field));
		}

		return result;
	}

	/**
	 * Die Funktion liefert den Wert des über den Index definierten Elements der
	 * Datenstruktur.<br>
	 * Wenn das benannte Element nicht existiert, wird ein
	 * <i>ContainerZugriffsFehler</i> geworfen.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.daten.container.Container#getContainerElement(int)
	 * @see #setContainerElement(int, Object)
	 */
	@Override
	public Object getContainerElement(final int index) {
		Object result = null;
		try {
			int accessIndex = index;
			if (accessIndex < 0) {
				accessIndex = definition.getFields().length + 1 + accessIndex;
			}
			final String name = definition.getFields()[accessIndex - 1];
			result = getStrukturElement(name);
		} catch (final IndexOutOfBoundsException e) {
			result = new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX, e.getMessage());
		}

		return result;
	}

	/**
	 * liefert die Definition, die der Struktur zu Grunde liegt.
	 *
	 * @return die Definition
	 */
	public DatenStrukturDefinition getDefinition() {
		return definition;
	}

	/**
	 * Die Funktion liefert den Wert des über den Namen definierten Elements der
	 * Datenstruktur.<br>
	 * Wenn das benannte Element nicht existiert, wird ein ObjektZugriffsFehler
	 * geworfen.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#getStrukturElement(java.lang.String)
	 * @see #setStrukturElement(String, Object)
	 */
	@Override
	public Object getStrukturElement(final String name) {
		if ((name == null) || (name.trim().length() == 0)) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
					"Der Zugriff auf ein ELement erfordert einen sinnvollen Namen");
		}
		final String accesName = name.trim();
		if (fields.containsKey(accesName)) {
			return fields.get(accesName);
		}
		throw new ObjektZugriffsFehler(UdaFehlerSubtyp.ELEMENT, "Die Datenstruktur enthält das folgende Element nicht: "
				+ accesName);
	}

	/**
	 * Die Funktion setzt den Wert des über den Index Namen definierten Elements
	 * der Datenstruktur auf den übergebenen Wert.<br>
	 * Wenn das benannte Element nicht existiert, wird ein
	 * <i>ContainerZugriffsFehler</i> geworfen.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getContainerElement(int)
	 */
	@Override
	public void setContainerElement(final int index, final Object daten) {
		try {
			int accessIndex = index;
			if (accessIndex < 0) {
				accessIndex = definition.getFields().length + 1 + accessIndex;
			}
			final String name = definition.getFields()[accessIndex - 1];
			setStrukturElement(name, daten);
		} catch (final IndexOutOfBoundsException e) {
			throw new ContainerZugriffsFehler(UdaFehlerSubtyp.INDEX, e.getMessage());
		}
	}

	/**
	 * Die Funktion setzt den Wert des über den Namen definierten Elements der
	 * Datenstruktur auf den übergebenen Wert.<br>
	 * Wenn das benannte Element nicht existiert, wird ein ObjektZugriffsFehler
	 * geworfen.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.ausdruck.Struktur#setStrukturElement(java.lang.String,
	 *      java.lang.Object)
	 * @see #getStrukturElement(String)
	 */
	@Override
	public void setStrukturElement(final String name, final Object daten) {
		final Object oldData = getStrukturElement(name);
		final Object destData = UdaInterpreter.getZuweisbareDaten(oldData, daten);
		if (destData == null) {
			throw new ArgumentFehler(UdaFehlerSubtyp.ARGUMENTE,
					"Zuweisung von Daten des Typs \""
							+ daten.getClass().getSimpleName() + "\" auf ein Element vom Typ \""
							+ oldData.getClass().getSimpleName() + "\" nicht erlaubt");
		}
		fields.put(name.trim(), destData);
	}

	/**
	 * liefert eine Zeichenkette, die die Datenstruktur repräsentiert.<br>
	 * Die Ausgabe erfolgt in der Form
	 * "{&lt;attributwert1&gt;,&lt;attributwert2&gt;,...}" {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		final StringBuffer buffer = new StringBuffer("{");
		for (final String element : definition.getFields()) {
			if (buffer.length() > 1) {
				buffer.append(",");
			}
			buffer.append(element);
			buffer.append('=');
			buffer.append(fields.get(element));
		}
		buffer.append('}');
		return buffer.toString();
	}
}
