/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.struktur;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;

/**
 * Testklasse zum Test der Klasse {@link DatenStrukturDefinition}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DatenStrukturDefinitionTest {

	/**
	 * die {@link DatenStrukturDefinition}, mit der getestet wird.
	 */
	private final DatenStrukturDefinition definition = new DatenStrukturDefinition(new String[] { "x", "y", "z" });

	/**
	 * testet den Konstruktor ohne Argumente.
	 */
	@Test(expected = ArgumentFehler.class)
	public void testKonstruktorNullFeld() {
		new DatenStrukturDefinition(null);
	}

	/**
	 * testet den Konstruktor mit einem leeren Feld.
	 */
	@Test(expected = ArgumentFehler.class)
	public void testKonstruktorLeeresFeld() {
		new DatenStrukturDefinition(new String[0]);
	}

	/**
	 * testet den Konstruktor mit doppelt vergebenen Namen in der Liste der
	 * Attribute.
	 */
	@Test(expected = ArgumentFehler.class)
	public void testKonstruktorDoppelteAttribute() {
		new DatenStrukturDefinition(new String[] { "x", " x  ", "     x   " });
	}

	/**
	 * testet den Konstruktor mit ungültigen Attributnamen.
	 */
	@Test
	public void testKonstruktorFalscheAttributNamen() {
		final String[] testNamen = { "Ein Attribut", "1abc" };
		for (final String eintrag : testNamen) {
			try {
				new DatenStrukturDefinition(new String[] { eintrag, "y", "z" });
				Assert.fail("Ungültiger Attributname zugelassen: " + eintrag);
			} catch (final ArgumentFehler e) {
				// Ungültiger Name abgelehnt
			}
		}
	}

	/**
	 * testet die Umwandlung der Definition in eine Zeichenkette.
	 */
	@Test
	public void testToString() {
		Assert.assertEquals("{x,y,z}", definition.toString());
	}

	/**
	 * testet das Auslesen der Strukturelemente.
	 */
	@Test
	public void testGetFields() {
		Assert.assertTrue(3 == definition.getFields().length);
		Assert.assertEquals("x", definition.getFields()[0]);
		Assert.assertEquals("y", definition.getFields()[1]);
		Assert.assertEquals("z", definition.getFields()[2]);
	}

	/**
	 * testet das Auslesen der Strukturelemente per Containerzugriff.
	 */
	@Test
	public void testGetContainerElement() {
		Assert.assertEquals("x", definition.getContainerElement(1));
		Assert.assertEquals("y", definition.getContainerElement(2));
		Assert.assertEquals("z", definition.getContainerElement(3));
		Assert.assertTrue(definition.getContainerElement(0) instanceof ContainerZugriffsFehler);
		Assert.assertTrue(definition.getContainerElement(-1) instanceof ContainerZugriffsFehler);
		Assert.assertTrue(definition.getContainerElement(99) instanceof ContainerZugriffsFehler);
	}

	/**
	 * testet das Setzen eines Attributs.
	 */
	@Test(expected = ArgumentFehler.class)
	public void testSetContainerElement() {
		definition.setContainerElement(1, "Neues Attribut");
	}
}
