/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.struktur;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ObjektZugriffsFehler;

/**
 * Testet die Funktionen der Klasse {@link DatenStruktur}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class DatenStrukturTest {

	/**
	 * die verwendete Datenstrukturdefinition.
	 */
	private final DatenStrukturDefinition definition = new DatenStrukturDefinition(new String[] { "x", "y", "z" });

	/**
	 * Aufruf des Konstruktors mit zu vielen Argumenten.
	 */
	@Test(expected = ArgumentFehler.class)
	public void testKonstruktorZuVieleInitialwerte() {
		Assert.assertEquals("{x=1,y=2,z=3}", new DatenStruktur(definition,
				new Object[] { 1, 2, 3, 4 }).toString());
	}

	/**
	 * Umwandlung der Datenstruktur in eine Zeichenkette.
	 */
	@Test
	public void testToString() {
		Assert.assertEquals("{x=undefiniert,y=undefiniert,z=undefiniert}",
				new DatenStruktur(definition, null).toString());
		Assert.assertEquals("{x=undefiniert,y=undefiniert,z=undefiniert}",
				new DatenStruktur(definition, new Object[] {}).toString());
		Assert.assertEquals("{x=1,y=undefiniert,z=undefiniert}",
				new DatenStruktur(definition, new Object[] { 1 }).toString());
		Assert.assertEquals("{x=1,y=2,z=undefiniert}", new DatenStruktur(definition,
				new Object[] { 1, 2 }).toString());
		Assert.assertEquals("{x=1,y=2,z=3}", new DatenStruktur(definition,
				new Object[] { 1, 2, 3 }).toString());
	}

	/**
	 * Auslesen eines Elements der Struktur.
	 */
	@Test
	public void testGetStrukturElement() {
		final DatenStruktur ds = new DatenStruktur(definition, new Object[] { 1, "Hallo", 3 });
		Assert.assertTrue(1 == ((Number) ds.getStrukturElement(" x ")).intValue());
		Assert.assertEquals("Hallo", ds.getStrukturElement("y "));
		Assert.assertTrue(3 == ((Number) ds.getStrukturElement("z   ")).intValue());

		try {
			Assert.assertEquals("Hallo", ds.getStrukturElement(null));
		} catch (final ArgumentFehler e) {
			// ungültiges Argument
		}
		try {
			Assert.assertEquals("Hallo", ds.getStrukturElement(""));
		} catch (final ArgumentFehler e) {
			// ungültiges Argument
		}

		try {
			Assert.assertEquals("Hallo", ds.getStrukturElement("  "));
		} catch (final ArgumentFehler e) {
			// ungültiges Argument
		}

		try {
			Assert.assertEquals("Hallo", ds.getStrukturElement("a"));
		} catch (final ObjektZugriffsFehler e) {
			// Element existiert nicht
		}
	}

	/**
	 * Setzen eines Elements der Struktur.
	 */
	@Test
	public void testSetStrukturElement() {
		final DatenStruktur ds = new DatenStruktur(definition, new Object[] { 1, "Hallo", 3 });
		ds.setStrukturElement(" x ", 7);
		Assert.assertTrue(7 == ((Number) ds.getStrukturElement(" x ")).intValue());

		try {
			ds.setStrukturElement(" x ", "Heinz");
			Assert.fail("Zuweisung eines String auf ein Element vom Typ Number möglich");
		} catch (final ArgumentFehler e) {
			// Zuweisung nicht ausgeführt
		}

		ds.setStrukturElement(" y ", "Heinz");
		Assert.assertEquals("Heinz", ds.getStrukturElement(" y "));

		ds.setStrukturElement(" y ", 24);
		Assert.assertEquals("24", ds.getStrukturElement(" y "));

		try {
			ds.setStrukturElement(null, 23);
			Assert.fail("Zuweisung ohne Argumentname erlaubt");
		} catch (final ArgumentFehler e) {
			// Zuweisung nicht ausgeführt
		}

		try {
			ds.setStrukturElement("   ", 23);
			Assert.fail("Zuweisung mit leerem Argumentname erlaubt");
		} catch (final ArgumentFehler e) {
			// Zuweisung nicht ausgeführt
		}

		try {
			ds.setStrukturElement("d", 23);
			Assert.fail("Zuweisung auf unbekanntes Attribut erlaubt");
		} catch (final ObjektZugriffsFehler e) {
			// Zuweisung nicht ausgeführt
		}
	}

	/**
	 * Auslesen eines Strukturelements über den Containerindex.
	 */
	@Test
	public void testGetContainerElement() {
		final DatenStruktur ds = new DatenStruktur(definition, new Object[] { 1, "Hallo", 3 });
		Assert.assertTrue(1 == ((Number) ds.getContainerElement(1)).intValue());
		Assert.assertEquals("Hallo", ds.getContainerElement(2));
		Assert.assertTrue(3 == ((Number) ds.getContainerElement(3)).intValue());
		Assert.assertEquals("Hallo", ds.getContainerElement(-2));

		Assert.assertTrue(ds.getContainerElement(0) instanceof ContainerZugriffsFehler);
		Assert.assertTrue(ds.getContainerElement(-7) instanceof ContainerZugriffsFehler);
		Assert.assertTrue(ds.getContainerElement(99) instanceof ContainerZugriffsFehler);
	}

	/**
	 * Setzen eines Containerelements testen.
	 */
	@Test
	public void testSetContainerElement() {
		final DatenStruktur ds = new DatenStruktur(definition, new Object[] { 1, "Hallo", 3 });
		ds.setContainerElement(1, 7);
		Assert.assertTrue(7 == ((Number) ds.getContainerElement(1)).intValue());

		try {
			ds.setContainerElement(1, "Heinz");
			Assert.fail("Zuweisung eines String auf ein Element vom Typ Number möglich");
		} catch (final ArgumentFehler e) {
			// Zuweisung nicht ausgeführt
		}

		ds.setContainerElement(2, "Heinz");
		Assert.assertEquals("Heinz", ds.getContainerElement(2));

		ds.setContainerElement(-2, 99);
		Assert.assertEquals("99", ds.getContainerElement(2));

		ds.setContainerElement(2, 24);
		Assert.assertEquals("24", ds.getContainerElement(2));

		try {
			ds.setContainerElement(0, 23);
			Assert.fail("Zuweisung auf Index 0 erlaubt");
		} catch (final ContainerZugriffsFehler e) {
			// Zuweisung nicht ausgeführt
		}

		try {
			ds.setContainerElement(-4, 23);
			Assert.fail("Zuweisung auf Index -4 erlaubt");
		} catch (final ContainerZugriffsFehler e) {
			// Zuweisung nicht ausgeführt
		}

		try {
			ds.setContainerElement(99, 23);
			Assert.fail("Zuweisung auf Index 99 erlaubt");
		} catch (final ContainerZugriffsFehler e) {
			// Zuweisung nicht ausgeführt
		}
	}

	/**
	 * Definition einer Datenstruktur ermitteln testen.
	 */
	@Test
	public void testGetDefinition() {
		final DatenStruktur ds = new DatenStruktur(definition, new Object[] { 1, "Hallo", 3 });
		Assert.assertEquals(definition, ds.getDefinition());
	}

}
