/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung;

import java.io.Externalizable;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.uda.uda.data.Historie;
import de.bsvrz.ibv.uda.uda.data.Skript;
import de.bsvrz.ibv.uda.uda.data.SkriptZustand;
import de.bsvrz.ibv.uda.uda.data.UDA;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Repräsentation eines Skript-Objekt innerhalb des Verwaltungsmoduls der
 * "Umfassenden Datenanalyse".<br>
 * Das Objekt repräsentiert die Daten des Skriptobjekts empfängt Änderungen
 * dieser Daten und publiziert den aktuellen Zustand über die entsprechenden
 * Attributgruppen der Datenverteilerkonfiguration.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public final class ServerSkript extends Skript
implements ClientSenderInterface, ClientReceiverInterface, Externalizable {
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * die Felder zur Serialisierung der Elemente des Objekts.<br>
	 *
	 * @author peuker
	 */
	private enum SkriptFeld {
		/**
		 * Datenfeld für den Bearbeitungszustand des Skripts.
		 */
		ZUSTAND, /**
		 * Datenfeld für das Erstellungsdatum.
		 */
		ERSTELLT_AM, /**
		 * Datenfeld für den Namen des Erstellers.
		 */
		ERSTELLT_DURCH, /**
		 * Datenfeld für die Beschreibung des Skripts.
		 */
		BESCHREIBUNG, /**
		 * Datenfeld für den Name des Skripts.
		 */
		NAME, /**
		 * Datenfeld für den Quelltext.
		 */
		QUELLE, /**
		 * Datenfeld für die Historie.
		 */
		HISTORIE
	}

	/**
	 * Versions-ID für Serialisierung.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * die Date in der die Skriptdaten abgelegt sind.
	 */
	private File skriptDatei;

	/**
	 * das Modul, in dem das Skript verwaltet wird.
	 */
	private final ServerUdaModul modul;

	/**
	 * das Systemobjekt, welches das Skriptobjekt innerhalb der
	 * Datenverteilerkonfiguration repräsentiert.<br>
	 * Über die Attributgruppen des Objekts werden Informationen zu dem
	 * betreffenden Skript ausgetauscht.
	 */
	private final SystemObject skriptObjekt;

	/**
	 * die Datenbeschreibung zum Publizieren der aktuellen Daten eines Skripts.
	 */
	private DataDescription istDesc;

	/**
	 * die Datenveschreibung zum Empfang von neuen Daten zur Beschreibung eines
	 * Skripts.
	 */
	private DataDescription vorgabeDesc;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein Skriptobjekt innerhalb der "Umfassenden
	 * Datenanalyse" für das angegebene Modul und das übergebene Systemobjekt.
	 *
	 * @param modul
	 *            das UdaModul
	 * @param obj
	 *            das Systemobjekt
	 * @throws UdaServerException
	 *             das Skriptobjekt konnte nicht angelegt werden
	 */
	public ServerSkript(final ServerUdaModul modul, final SystemObject obj) throws UdaServerException {
		super(obj.getNameOrPidOrId());

		this.modul = modul;
		this.skriptObjekt = obj;

		skriptDatei = new File(modul.getModulVerzeichnis(), "skripte");
		if (!skriptDatei.exists()) {
			skriptDatei.mkdirs();
		}
		skriptDatei = new File(skriptDatei, obj.getPid());

		initialisiereVerbindung();
	}

	/**
	 * aktualisiert die Daten des Skripts entsprechend dem empfangenen
	 * Datensatz.<br>
	 * Alle internen Daten der Skriptdefinition werden mit den empfangenen Daten
	 * überschrieben. Eine Ausnahme bildet der Eintrag für die Skripthistorie,
	 * dieser wird als neuer Eintrag zu den bestehenden hinzugefügt.<br>
	 * Nach der internen Aktualisierung werden die Daten persistent gespeichert.
	 *
	 * @param data
	 *            die neuen Daten
	 * @throws IOException
	 *             die Daten konnten nicht erfolgreich aktualisiert werden
	 */
	private void aktualisiereDaten(final Data data) throws IOException {
		setSkriptName(data.getTextValue(UDA.SKRIPT_ATT_NAME).getText());
		setSkriptBeschreibung(data.getTextValue(UDA.SKRIPT_ATT_BESCHREIBUNG).getText());
		setErstelltAm(data.getTimeValue(UDA.SKRIPT_ATT_ERSTELLT_AM).getMillis());
		setErstelltDurch(data.getTextValue(UDA.SKRIPT_ATT_ERSTELLT_DURCH).getText());

		final Data historieData = data.getArray(UDA.SKRIPT_ATT_VERSIONSHISTORIE).getItem(0);
		if (historieData != null) {
			final long zeitPunkt = historieData.getTimeValue(UDA.ATL_HISTORIE_ATT_AENDERUNGSDATUM).getMillis();
			final String name = historieData.getTextValue(UDA.ATL_HISTORIE_ATT_NAME).getText();
			final String grund = historieData.getTextValue(UDA.ATL_HISTORIE_ATT_GRUND).getText();

			if ((name != null) && (name.length() > 0)) {
				addHistorie(new Historie(zeitPunkt, name, grund));
			}
		}

		setZustand(SkriptZustand.getZustand(data.getUnscaledValue(UDA.SKRIPT_ATT_ZUSTAND).intValue()));

		final Data.Array code = data.getArray(UDA.SKRIPT_ATT_QUELLTEXT);
		clearSourceCode();
		for (int i = 0; i < code.getLength(); i++) {
			addSourceLine(code.getTextValue(i).getText());
		}
		sichereDaten(skriptDatei);
	}

	/**
	 * {@inheritDoc}.<br>
	 * Implementierung der Sendesteuerung für das Objekt.<br>
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * ClientSenderInterface. Eine Sendeanforderung wird ausgelöst, wenn es
	 * Empfänger für die aktuellen Daten des Skripts gibt. Die Funktion
	 * übernimmt dann die Veröffentlichung der Daten.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientSenderInterface#dataRequest(de.bsvrz.dav.daf.main.config.SystemObject,
	 *      de.bsvrz.dav.daf.main.DataDescription, byte)
	 */
	@Override
	public void dataRequest(final SystemObject object, final DataDescription desc, final byte status) {
		if (status == ClientSenderInterface.START_SENDING) {
			try {
				publiziereDaten();
			} catch (final SendSubscriptionNotConfirmed e) {
				// kann eigentlich nicht passieren, da ja hier gerade die
				// Sendebstätigung erfolgt ist.
				e.printStackTrace();
			}
		}
	}

	/**
	 * Anlegen und Füllen eines Datensatzes zum Veröffentlichen der aktuellen
	 * Skriptdaten.
	 *
	 * @return den Datensatz mit den Daten der Skriptdefinition
	 */
	public Data fuelleDatensatz() {
		final Data data = modul.getVerbindung().createData(istDesc.getAttributeGroup());
		data.getTextValue(UDA.SKRIPT_ATT_NAME).setText(getName());
		data.getTextValue(UDA.SKRIPT_ATT_BESCHREIBUNG).setText(getBeschreibung());
		data.getTimeValue(UDA.SKRIPT_ATT_ERSTELLT_AM).setMillis(getErstellungsDatum());
		data.getTextValue(UDA.SKRIPT_ATT_ERSTELLT_DURCH).setText(getErsteller());

		final Data.Array historie = data.getArray(UDA.SKRIPT_ATT_VERSIONSHISTORIE);
		historie.setLength(getHistorie().size());
		for (int idx = 0; idx < getHistorie().size(); idx++) {
			final Historie eintrag = getHistorie().get(idx);
			historie.getItem(idx).getTimeValue(UDA.ATL_HISTORIE_ATT_AENDERUNGSDATUM).setMillis(eintrag.getZeitPunkt());
			historie.getItem(idx).getTextValue(UDA.ATL_HISTORIE_ATT_NAME).setText(eintrag.getName());
			historie.getItem(idx).getTextValue(UDA.ATL_HISTORIE_ATT_GRUND).setText(eintrag.getGrund());
		}

		data.getUnscaledValue(UDA.SKRIPT_ATT_ZUSTAND).set(getZustand().getCode());

		final Data.Array code = data.getArray(UDA.SKRIPT_ATT_QUELLTEXT);
		code.setLength(getSourceCode().size());
		for (int idx = 0; idx < getSourceCode().size(); idx++) {
			code.getTextValue(idx).setText(getSourceCode().get(idx));
		}

		return data;
	}

	/**
	 * liefert des Skriptobjekt, das das Skript innerhalb der Konfiguration des
	 * Datenverteilers repräsentiert.
	 *
	 * @return das Objekt
	 */
	@Override
	public SystemObject getDavObjekt() {
		return skriptObjekt;
	}

	/**
	 * liefert das Modul, dem das Skript zugeordnet ist.
	 *
	 * @return das Modul
	 */
	public ServerUdaModul getModul() {
		return modul;
	}

	/**
	 * stellt die Verbindungen zu Datenverteiler her.<br>
	 * Das Objekt meldet sich als Quelle für den Istzustand der Daten eines
	 * Skripts und als Senke für den Empfang neuer Vorgabedaten zur Definition
	 * des Skripts an.
	 *
	 * @throws UdaServerException
	 *             die Anmeldung konnte nicht ausgeführt werden.
	 */
	private void initialisiereVerbindung() throws UdaServerException {
		try {
			ladeDaten(skriptDatei);
		} catch (final IOException e) {
			throw new UdaServerException("Skript: " + getName()
			+ " konnte nicht initialisiert werden", e);
		}
		try {
			final ClientDavInterface dav = modul.getVerbindung();
			final DataModel model = dav.getDataModel();
			istDesc = new DataDescription(model.getAttributeGroup(UDA.SKRIPT_ATG_SKRIPT), model.getAspect(UDA.ASP_IST));
			vorgabeDesc = new DataDescription(model.getAttributeGroup(UDA.SKRIPT_ATG_SKRIPT),
					model.getAspect(UDA.ASP_VORGABE));
			dav.subscribeReceiver(this, skriptObjekt, vorgabeDesc, ReceiveOptions.normal(), ReceiverRole.drain());
			dav.subscribeSender(this, skriptObjekt, istDesc, SenderRole.source());
		} catch (final OneSubscriptionPerSendData e) {
			throw new RuntimeException("Skript: " + getName()
			+ " konnte nicht initialisiert werden", e);
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Implementierung der entsprechenden Funktion der Schnittstelle
	 * ClientSenderInterface. Die Sendesteuerung wird für alle
	 * Datenspezifikationen für die eine Anmeldung als Sender erfolgte
	 * unterstützt.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientSenderInterface#isRequestSupported(de.bsvrz.dav.daf.main.config.SystemObject,
	 *      de.bsvrz.dav.daf.main.DataDescription)
	 */
	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription desc) {
		return true;
	}

	/**
	 * lädt die Daten des Objekts aus der angegebenen Datei.
	 *
	 * @param file
	 *            die Datei
	 * @throws IOException
	 *             die Daten konnten nicht eingelesen werden
	 */
	private void ladeDaten(final File file) throws IOException {
		if (file.exists()) {
			try (ObjectInputStream input = new ObjectInputStream(new FileInputStream(file))) {
				readExternal(input);
			} catch (final ClassNotFoundException e) {
				final String msg = "Inkompatible Daten aus Skriptdatei: " + file.getAbsolutePath()
				+ " können nicht gelesen werden";
				ServerSkript.LOGGER.error(msg, e);
				throw new RuntimeException(msg);
			}
		}
	}

	/**
	 * veröffentlicht die aktuellen Daten des Skripts.<br>
	 * Die Funktion sichert den aktuellen Zustand
	 *
	 * @throws SendSubscriptionNotConfirmed
	 * @throws SendSubscriptionNotConfirmed
	 *             die Sendeanmeldung für die Daten ist nicht erfolgt
	 */
	private void publiziereDaten() throws SendSubscriptionNotConfirmed {
		final Data data = fuelleDatensatz();
		modul.getVerbindung().sendData(new ResultData(skriptObjekt, istDesc, System.currentTimeMillis(), data));
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void readExternal(final ObjectInput in) throws IOException, ClassNotFoundException {

		final Map<SkriptFeld, Object> daten = (Map<SkriptFeld, Object>) in.readObject();

		setSkriptName((String) daten.get(SkriptFeld.NAME));
		setSkriptBeschreibung((String) daten.get(SkriptFeld.BESCHREIBUNG));
		setErstelltAm((Long) daten.get(SkriptFeld.ERSTELLT_AM));
		setErstelltDurch((String) daten.get(SkriptFeld.ERSTELLT_DURCH));

		clearHistorie();
		getHistorie().addAll((List<Historie>) daten.get(SkriptFeld.HISTORIE));

		setZustand(SkriptZustand.getZustand((Integer) daten.get(SkriptFeld.ZUSTAND)));

		clearSourceCode();
		getSourceCode().addAll((List<String>) daten.get(SkriptFeld.QUELLE));
	}

	/**
	 * entfernt die Daten des Skripts aus dem Dateisystem des Servers der
	 * "Umfassenden Datenanalyse".
	 *
	 * @throws ConfigurationChangeException
	 *             beim Entfernen des Objekts aus der
	 *             Datenverteilerkonfiguration ist ein Fehler aufgetreten
	 */
	void remove() throws ConfigurationChangeException {
		skriptDatei.delete();
		skriptObjekt.invalidate();
	}

	/**
	 * sichert die Daten des Objekts in die übergebene Datei.
	 *
	 * @param file
	 *            die Datei
	 * @throws IOException
	 *             die Sicherung konnte nicht ausgeführt werden
	 */
	private void sichereDaten(final File file) throws IOException {
		try (ObjectOutputStream output = new ObjectOutputStream(new FileOutputStream(file))) {
			writeExternal(output);
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion empfängt die Vorgabedaten für die Definition des Skripts,
	 * aktualisiert die Daten intern entsprechend und publiziert den neuen
	 * Zustand über die entsprechende Datenspezifikation.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(final ResultData[] resultList) {
		for (final ResultData result : resultList) {
			if (result.hasData()) {
				final DataDescription desc = result.getDataDescription();
				if (desc.getAttributeGroup().equals(vorgabeDesc.getAttributeGroup())) {
					if (desc.getAspect().equals(vorgabeDesc.getAspect())) {
						try {
							aktualisiereDaten(result.getData());
							publiziereDaten();
						} catch (final IOException | SendSubscriptionNotConfirmed e) {
							final String message = "Skriptdaten konnten im Dateisystem nicht ausgelagert werden";
							ServerSkript.LOGGER.error(message, e);
							throw new RuntimeException(message);
						}
					}
				}
			}
		}
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
	 */
	@Override
	public void writeExternal(final ObjectOutput out) throws IOException {

		final Map<SkriptFeld, Object> daten = new HashMap<>();

		daten.put(SkriptFeld.NAME, getName());
		daten.put(SkriptFeld.BESCHREIBUNG, getBeschreibung());
		daten.put(SkriptFeld.ERSTELLT_AM, getErstellungsDatum());
		daten.put(SkriptFeld.ERSTELLT_DURCH, getErsteller());
		daten.put(SkriptFeld.HISTORIE, getHistorie());
		daten.put(SkriptFeld.ZUSTAND, getZustand().getCode());
		daten.put(SkriptFeld.QUELLE, getSourceCode());

		out.writeObject(daten);
	}

}
