/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung.protokoll;

import java.io.Serializable;

import com.bitctrl.Constants;

import de.bsvrz.ibv.uda.uda.data.SkriptLauf;

/**
 * die abstrake Implementierung eines Protokollobjekts.<br>
 * Protokolle beschreiben, in welcher Art und Weise die Ausgaben eines
 * Uda-Skripts veröffentlicht werden sollen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class Protokoll implements Serializable {

	/**
	 * Versions-ID für die Serialisierung.
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * der Typ des Protokolls.
	 */
	private final ProtokollTyp typ;

	/**
	 * die Anzahl der zu erstellenden Ausgabeobjekte.
	 */
	private long anzahl;

	/**
	 * die maximale Größe eines Ausgabeobjekts.
	 */
	private long maxSize;

	/**
	 * der Name für das Ausgabeobjekt.
	 */
	private String name;

	/**
	 * Konstruktr.<br>
	 * Die Funjtion erzeugt ein Protokollobjekt des entsprechenden Typs.
	 *
	 * @param typ
	 *            der Typ des Protokolls
	 */
	protected Protokoll(final ProtokollTyp typ) {
		this.typ = typ;
	}

	/**
	 * schließt die Ausgabekanäle des entsprechenden Protokolls.
	 */
	public abstract void abschliessen();

	/**
	 * gibt die übergebenen Daten entsprechend der Protokollimplementierung aus.
	 *
	 * @param daten
	 *            die auszugebenden Daten
	 */
	public abstract void ausgeben(ProtokollEintrag daten);

	/**
	 * liefert die maximale Anzahl der zu erstellenden Ausgabeobjekte.
	 *
	 * @return die Anzahl
	 * @see #setAnzahl(long)
	 */
	public long getAnzahl() {
		return anzahl;
	}

	/**
	 * liefert die maximale Größe eines zu erstellenden Ausgabeobjekts.
	 *
	 * @return die maximale Größe
	 * @see #setMaxSize(long)
	 */
	public long getMaxSize() {
		return maxSize;
	}

	/**
	 * liefert den Namen des zu erstellenden Ausgabeobjekts.
	 *
	 * @return den Name
	 * @see #setName(String)
	 */
	public String getName() {
		if (name == null) {
			name = Constants.EMPTY_STRING;
		}
		return name;
	}

	/**
	 * liefert den Typ des zu erstellenden Protokolls.
	 *
	 * @return den Typ
	 */
	public ProtokollTyp getTyp() {
		return typ;
	}

	/**
	 * initialisiert die Ausgabekanäle des entsprechenden Protokolls.
	 *
	 * @param lauf
	 *            der Skriptlauf f&uuml;r den die Ausgaben erfolgen sollen
	 */
	public abstract void initialisiere(SkriptLauf lauf);

	/**
	 * setzt die maximale Anzahl der zu erstellenden Ausgabeobjekte.
	 *
	 * @param anzahl
	 *            die Anzahl
	 * @see #getAnzahl()
	 */
	public void setAnzahl(final long anzahl) {
		this.anzahl = anzahl;
	}

	/**
	 * setzt die maximale Größe eines zu erstellenden Ausgabeobjekts.
	 *
	 * @param groesse
	 *            die Größe
	 * @see #getMaxSize()
	 */
	public void setMaxSize(final long groesse) {
		this.maxSize = groesse;
	}

	/**
	 * setzt den Namen des zu erstellenden Ausgabeobjekts.
	 *
	 * @param name
	 *            der Name
	 * @see #getName()
	 */
	public void setName(final String name) {
		if (name.contains(",")) {
			throw new IllegalArgumentException("Der Name eines Protokolls darf kein Komma enthalten");
		}
		this.name = name;
	}
}
